<?php

namespace Drupal\y_branch_menu\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Http\RequestStack;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides Branch Menu block.
 *
 * @Block(
 *   id = "y_branch_menu",
 *   admin_label = @Translation("Branch Menu Block"),
 *   category = @Translation("Branch blocks")
 * )
 */
class BranchMenuBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Cuurent node.
   *
   * @var \Drupal\node\Entity\Node
   */
  protected $entity;

  /**
   * The current path.
   *
   * @var string
   */
  private string $currentPath;

  /**
   * Constructs a new EventInfoBlock instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param CurrentRouteMatch $currentRouteMatch
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, CurrentRouteMatch $currentRouteMatch, RequestStack $request_stack) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentRouteMatch = $currentRouteMatch;
    $this->entity = $currentRouteMatch->getParameter('node');
    $this->currentPath = $request_stack->getCurrentRequest()->getPathInfo();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('request_stack'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'home_link' => TRUE,
      'limit_links' => 6,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    if (!$node = $this->entity) {
      return [];
    }

    switch ($node->getType()) {
      case 'article_lb':
      case 'lb_event':
        if (!$node->hasField('field_location_reference') ||
          $node->get('field_location_reference')->isEmpty()) {
          break;
        }
        $referencedEntities = $node->get('field_location_reference')->referencedEntities();
        $node = reset($referencedEntities);
      case 'branch':
        if (!$node->hasField('field_branch_menu_links')) {
          break;
        }
        $fieldLinks = $node->get('field_branch_menu_links');
    }

    if (empty($fieldLinks) || $fieldLinks->isEmpty()) {
      return [];
    }

    $config = $this->configuration;
    $result = [];

    if ($config['home_link']) {
      $result[] = $this->getHomeLink($node);
      $config['limit_links']--;
    }

    array_push($result, ...$this->getLinks($fieldLinks, $config['limit_links']));
    return ['#links' => $result];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;

    $form['home_link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show "Home" link'),
      '#default_value' => $config['home_link'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['home_link'] = $form_state->getValue('home_link');
  }

  /**
   * Build home link which redirect to branch page.
   *
   * @param \Drupal\node\NodeInterface $node
   *
   * @return array|Link
   */
  private function getHomeLink($node) {
    $url = $node->toUrl('canonical', ['attributes' => ['class' => ['home']]]);
    if ($url->toString() === $this->currentPath) {
      $url->mergeOptions(['attributes' => ['class' => ['active']]]);
    }
    return Link::fromTextAndUrl($this->t('Branch Home'), $url);
  }

  /**
   * @param $fieldBranchMenuLinks
   * @param int $limit number of links returned
   * @return array links
   */
  private function getLinks($fieldBranchMenuLinks, int $limit) {
    $result = [];
    if ($values = $fieldBranchMenuLinks->getValue()) {
      foreach ($values as $value) {
        if (count($result) >= $limit) {
          break;
        }
        $url = Url::fromUri($value['uri'], $value['options']);
        if ($url->toString() === $this->currentPath) {
          $url->mergeOptions(['attributes' => ['class' => ['active']]]);
        }
        $result[] = Link::fromTextAndUrl($value['title'], $url);
      }
    }
    return $result;
  }
}
